@@ -259,10 +259,6 @@ search_pager_init (YelpSearchPager *pager)
 static void
 search_pager_dispose (GObject *object)
 {
-    YelpSearchPager *pager = YELP_SEARCH_PAGER (object);
-
-    g_free (pager->priv);
-
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -420,6 +416,15 @@ typedef struct
     xmlNode *node;
 } SnippetLocation;
 
+static void snippet_closed   (BeagleSnippetRequest  *request,
+			      SnippetLocation       *snippet_location);
+static void snippet_response (BeagleSnippetRequest  *request,
+			      BeagleSnippetResponse *response,
+			      SnippetLocation       *snippet_location);
+static void snippet_error    (BeagleSnippetRequest  *request,
+			      GError                *error,
+			      SnippetLocation       *snippet_location);
+
 static void
 snippet_closed (BeagleSnippetRequest *request, SnippetLocation *snippet_location)
 {
@@ -429,6 +434,17 @@ snippet_closed (BeagleSnippetRequest *request, SnippetLocation *snippet_location
 
     pager->priv->snippet_request_count --;
     check_finished (pager);
+
+    g_signal_handlers_disconnect_by_func (request,
+					  G_CALLBACK (snippet_response),
+					  snippet_location);
+    g_signal_handlers_disconnect_by_func (request,
+					  G_CALLBACK (snippet_error),
+					  snippet_location);
+    g_signal_handlers_disconnect_by_func (request,
+					  G_CALLBACK (snippet_closed),
+					  snippet_location);
+
     g_free (snippet_location);
     g_object_unref (request);
 }
@@ -444,7 +460,7 @@ snippet_response (BeagleSnippetRequest *request, BeagleSnippetResponse *response
 
     if (xml == NULL) {
 	d(g_print ("snippet_response empty\n"));
-	goto done;
+	return;
     }
     d(g_print ("snippet_response: %s\n", xml));
 
@@ -452,20 +468,17 @@ snippet_response (BeagleSnippetRequest *request, BeagleSnippetResponse *response
     snippet_doc = xmlParseDoc (BAD_CAST xmldoc);
     g_free (xmldoc);
     if (!snippet_doc)
-	goto done;
+	return;
     node = xmlDocGetRootElement (snippet_doc);
     xmlUnlinkNode (node);
     xmlAddChild (snippet_location->node, node);
     xmlFreeDoc (snippet_doc);
-    done:
-    snippet_closed (request, snippet_location);
 }
 
 static void
 snippet_error (BeagleSnippetRequest *request, GError *error, SnippetLocation *snippet_location)
 {
     d(g_print ("snippet_error\n"));
-    snippet_closed (request, snippet_location);
 }
 
 
@@ -542,7 +555,6 @@ finished_cb (BeagleQuery            *query,
     YelpSearchPagerPriv *priv = YELP_SEARCH_PAGER (pager)->priv;
 
     d(g_print ("finished_cb\n"));
-    g_object_unref (query);
 
     g_ptr_array_sort (priv->hits, compare_hits);
 
@@ -592,6 +604,14 @@ finished_cb (BeagleQuery            *query,
 					  NULL);
     }
 
+    g_signal_handlers_disconnect_by_func (query,
+					  G_CALLBACK (hits_added_cb),
+					  pager);
+    g_signal_handlers_disconnect_by_func (query,
+					  G_CALLBACK (finished_cb),
+					  pager);
+    g_object_unref (query);
+
     g_ptr_array_foreach (priv->hits, (GFunc) beagle_hit_unref, NULL);
     g_ptr_array_free (priv->hits, TRUE);
     priv->hits = NULL;
@@ -1155,7 +1175,7 @@ slow_search_setup (YelpSearchPager *pager)
 			    ptr - container->base_filename);
 	container->base_path = g_strdup (path);
 	container->search_term = 
-	    g_strsplit (g_utf8_casefold (pager->priv->search_terms, -1),
+	    g_strsplit (g_utf8_casefold (g_strstrip(pager->priv->search_terms), -1),
 			" ", -1);
 	container->search_status = NOT_SEARCHING;
 
